//---------------------------------------------------------------------------//
//
//	Secret Garden - Game Main
//
//	Date : 2008/05/05
//
//---------------------------------------------------------------------------//

#include "GameMain.h"

//---------------------------------------------------------------------------//
//	Game Main Translator Class
//---------------------------------------------------------------------------//

CGameMainTranslator::CGameMainTranslator()
{
	m_nSelectedCharacter = 0;
	m_nNextState = CGameState::GAME_MAIN;
}

CGameMainTranslator::~CGameMainTranslator()
{
}

//---------------------------------------------------------------------------//
//	Game Main Class
//---------------------------------------------------------------------------//

CGameMain::CGameMain()
{
}

CGameMain::~CGameMain()
{
}

bool CGameMain::Init(CGameStateTranslator* pTranslator)
{
	CGameMainTranslator* pData = static_cast<CGameMainTranslator*>(pTranslator);
	//m_pGameMainTranslate = new CGameMainTranslator(pData);

	m_pHero = new CHero;
	m_pHero->Init(pData->m_nSelectedCharacter);
	m_pEnemy = new CEnemy;
	m_pEnemy->Init();
	m_pMap = new CMap;
	m_pMap->Init();

	colWithEnemy = false;
	colWithRock = false;
	colWithRockMove = false;
	colWithGem = false;
	colWithKey = false;
	colWithDoorOpen = false;

	// ΰ hp-  
	m_herodead = false;
	m_tempScore = 0;

	
	return true;
}

void CGameMain::Release()
{
	SAFE_DELETE(m_pHero);
	SAFE_DELETE(m_pEnemy);
	SAFE_DELETE(m_pMap);
	//SAFE_DELETE(m_pGameMainTranslate);
}

int CGameMain::Update()
{
	if (m_herodead)
	{
		m_herodead = m_pHero->DeadMotion();
	}
	else 
	{

		if (DIKRepeat(DIK_UP))
		{	
			//! ?  ()
			m_pHero->SetDirection(m_pHero->UP);
			m_pHero->AddY(-4);
		}
		else if (DIKRepeat(DIK_DOWN))
		{
			//! ?  ()
			m_pHero->SetDirection(m_pHero->DOWN);
			m_pHero->AddY(4);
		}
		else if (DIKRepeat(DIK_LEFT))
		{
			//! ?  ()
			m_pHero->SetDirection(m_pHero->LEFT);
			m_pHero->AddX(-4);
		}
		else if (DIKRepeat(DIK_RIGHT))
		{
			//! ?  ()
			m_pHero->SetDirection(m_pHero->RIGHT);
			m_pHero->AddX(4);
		}

		if (DIKRepeat(DIK_R))
		{
			m_tempScore = 0;
			int tempScore = m_pHero->GetScore();
			m_pHero->Init();
			m_pHero->SetScore(tempScore);
			m_pEnemy->Init();
			m_pMap->Init();
		}

	}

	m_pHero->Update();
	
	m_pEnemy->Move();
	m_pEnemy->Update();
	m_pMap->Update();
	CollisionTest();

		return true;
}

void CGameMain::Render(CRenderer* pRenderer)
{
	pRenderer->BeginScene();
	pRenderer->BeginDrawTexture(1600, 1200);

	//! ٴڷ̾  (1 layer)
	m_pMap->Render(1, pRenderer);
	
	//! Ʈ ̾  (2 layer)
	m_pMap->Render(2, pRenderer);

	//! ĳ ̵
	m_pHero->Render(pRenderer);

	//!  ̵
	m_pEnemy->Render(pRenderer);

	//! Ʈ ̾  (3 layer)
	m_pMap->Render(3, pRenderer);

	pRenderer->EndDrawTexture();	// Texture ׸⸦  (׷   ׸  ġ Ҽ ִٳ)

	//! ? Ʈ  ׽Ʈ
	static TCHAR szBuff[256];
	_stprintf(szBuff, _T("HeroCR : %d, %d"), m_pHero->GetColRect()->x1, m_pHero->GetColRect()->y1);
	pRenderer->DrawTextDC(90, 15, szBuff, RGB(255, 255, 255));
	_stprintf(szBuff, _T("Map : %d, %d"), m_pMap->GetColRect(2,2,1)->x1, m_pMap->GetColRect(2,2,1)->y1);
	pRenderer->DrawTextDC(250, 15, szBuff, RGB(255, 255, 255));
	
	_stprintf(szBuff, _T("score : %d"), m_pHero->GetScore());
	pRenderer->DrawTextDC(350, 30, szBuff, RGB(255, 255, 0));
	_stprintf(szBuff, _T("tempscore : %d"), m_tempScore);
	pRenderer->DrawTextDC(350, 15, szBuff, RGB(255, 255, 0));

	_stprintf(szBuff, _T("HeroPos : %d %d"), m_pHero->GetX(), m_pHero->GetY());
	pRenderer->DrawTextDC(500, 10, szBuff, RGB(255, 255, 0));
	_stprintf(szBuff, _T("EnemyPos : %d %d"), m_pEnemy->GetX(), m_pEnemy->GetY());
	pRenderer->DrawTextDC(600, 5, szBuff, RGB(255, 255, 0));

	switch(m_pEnemy->GetDirection())
	{
	case m_pEnemy->LEFT:
		_stprintf(szBuff, _T("LEFT"));
		pRenderer->DrawTextDC(600, 15, szBuff, RGB(255, 255, 0));
		break;
	case m_pEnemy->RIGHT:
		_stprintf(szBuff, _T("RIGHT"));
		pRenderer->DrawTextDC(600, 15, szBuff, RGB(255, 255, 0));
		break;
	case m_pEnemy->UP:
		_stprintf(szBuff, _T("UP"));
		pRenderer->DrawTextDC(600, 15, szBuff, RGB(255, 255, 0));
		break;
	case m_pEnemy->DOWN:
		_stprintf(szBuff, _T("DOWN"));
		pRenderer->DrawTextDC(600, 15, szBuff, RGB(255, 255, 0));
		break;
	default:
		break;

	}
	if (colWithEnemy)
	{
		_stprintf(szBuff, _T("Crash"));
		pRenderer->DrawTextDC(500, 20, szBuff, RGB(255, 255, 255));
	}	
	
}

// ========================================================================
//! hero, enemy, wall  浹׽Ʈ  
void CGameMain::CollisionTest()		
{
	static int react = 10;		// ΰ Ÿϰ 浹 ݵ

	//! ٱ  浹ó
	if (m_pHero->GetX() < 0)
		m_pHero->AddX(4);
	else if (m_pHero->GetX() >= (MAP_WIDTH-1)*128)
		m_pHero->AddX(-4);
	else if (m_pHero->GetY() < 0)
		m_pHero->AddY(4);
	else if (m_pHero->GetY() >= (MAP_HEIGHT-1)*128)
		m_pHero->AddY(-4);

	//! ΰ,  浹ó
	if (m_pEnemy != NULL)		
	{
		colWithEnemy = false;
		colWithEnemy = IsCrash(m_pHero->GetColRect(), m_pEnemy->GetColRect());
		
		if (colWithEnemy)
			m_herodead = true;
	}

	//! ΰ, , , Ű 浹ó
	if (m_pMap != NULL)	
	{
		for (int i=0; i<MAP_HEIGHT; i++)
		{
			for (int j=0; j<MAP_WIDTH; j++)
			{
				//! Rock ó ( ϵ)
				colWithRock = false;
				if (m_pMap->GetTile(2,i,j) == m_pMap->ROCK)
				{
					colWithRock = IsCrash(m_pHero->GetColRect(), m_pMap->GetColRect(2,i,j));
				}
				if (colWithRock)
				{
					switch (m_pHero->GetDirection())
					{
						case m_pHero->LEFT :
							m_pHero->AddX(react);
							break;
						case m_pHero->RIGHT :
							m_pHero->AddX(-react);
							break;
						case m_pHero->UP :
							m_pHero->AddY(react);
							break;
						case m_pHero->DOWN :
							m_pHero->AddY(-react);
							break;
						default :
							break;
					}
				}

				//! RockMove ó
				colWithRockMove = false;
				if (m_pMap->GetTile(2,i,j) == m_pMap->ROCKMOVE)
				{
					colWithRockMove = IsCrash(m_pHero->GetColRect(), m_pMap->GetColRect(2,i,j));
				}
				if (colWithRockMove)	// ̵     
				{
					switch (m_pHero->GetDirection())
					{
						case m_pHero->LEFT :
							if (j == 0 || m_pMap->GetTile(2,i,j-1) != 0 || m_pMap->GetTile(3,i,j-1) != 0)
							{
								m_pHero->AddX(react);
								return;
							}
							m_pMap->RockMoveMotion(i,j,m_pHero->LEFT);
							if (m_pMap->EndMotion())
							{
								m_pMap->SetTile(2, i, j, m_pMap->BACKTEX);
								m_pMap->SetTile(2, i, j-1, m_pMap->ROCKMOVE);	
							}
							break;
						case m_pHero->RIGHT :
							if (j == MAP_WIDTH-1 || m_pMap->GetTile(2,i,j+1) != 0 || m_pMap->GetTile(3,i,j+1) != 0)
							{
								m_pHero->AddX(-react);
								return;
							}
							m_pMap->RockMoveMotion(i,j,m_pHero->RIGHT);
							if (m_pMap->EndMotion())
							{
								m_pMap->SetTile(2, i, j, m_pMap->BACKTEX);
								m_pMap->SetTile(2, i, j+1, m_pMap->ROCKMOVE);	
							}
							break;
						case m_pHero->UP :
							if (i == 0 || m_pMap->GetTile(2,i-1,j) != 0 || m_pMap->GetTile(3,i-1,j) != 0)
							{
								m_pHero->AddY(react);
								return;
							}
							m_pMap->RockMoveMotion(i,j,m_pHero->UP);
							if (m_pMap->EndMotion())
							{
								m_pMap->SetTile(2, i, j, m_pMap->BACKTEX);
								m_pMap->SetTile(2, i-1, j, m_pMap->ROCKMOVE);	
							}
							break;
						case m_pHero->DOWN :
							if (i == MAP_HEIGHT-1 || m_pMap->GetTile(2,i+1,j) != 0 || m_pMap->GetTile(3,i+1,j) != 0)
							{
								m_pHero->AddY(-react);
								return;
							}
							m_pMap->RockMoveMotion(i,j,m_pHero->DOWN);
							if (m_pMap->EndMotion())
							{
								m_pMap->SetTile(2, i, j, m_pMap->BACKTEX);
								m_pMap->SetTile(2, i+1, j, m_pMap->ROCKMOVE);	
							}
							break;
						default :
							break;
					}
				}

				//! Gem  浹
				colWithGem = false;
				if (m_pMap->GetTile(2,i,j) == m_pMap->GEMBLUE || m_pMap->GetTile(2,i,j) == m_pMap->GEMGREEN || m_pMap->GetTile(2,i,j) == m_pMap->GEMORANGE)
					colWithGem = IsCrash(m_pHero->GetColRect(), m_pMap->GetColRect(2,i,j));
				if (colWithGem)
				{
					switch (m_pMap->GetTile(2,i,j))
					{
						case m_pMap->GEMBLUE :
							m_tempScore += 100;
							break;
						case m_pMap->GEMGREEN :
							m_tempScore += 300;
							break;
						case m_pMap->GEMORANGE :
							m_tempScore += 500;
							break;
						default :
							break;
					}
					m_pMap->SetTile(2,i,j,m_pMap->BACKTEX);
				}

				//! Key  浹
				colWithKey = false;
				if (m_pMap->GetTile(2,i,j) == m_pMap->KEY)
					colWithKey = IsCrash(m_pHero->GetColRect(), m_pMap->GetColRect(2,i,j));
				if (colWithKey)
				{
					m_pMap->SetDoorState(true);
					m_pMap->SetTile(2,i,j,m_pMap->BACKTEX);
				}

				//! DOOROPEN  浹
				colWithDoorOpen = false;
				if (m_pMap->GetTile(2,i,j) == m_pMap->DOOROPEN)
					colWithDoorOpen = IsCrash(m_pHero->GetColRect(), m_pMap->GetColRect(2,i,j));
				if (colWithDoorOpen)
				{
					if (m_pMap->GetStageNumber() == 3)
					{
						m_pTranslator = new CGameStateTranslator;
						m_pTranslator->m_nPrevState = CGameState::GAME_MAIN;
						m_pTranslator->m_nNextState = CGameState::GAME_RESULT;
						m_pTranslator->Translate();
					}
					m_pMap->NextStage();
					m_pMap->Init();
					m_pHero->SetX(0);
					m_pHero->SetY(0);
					m_pHero->AddScore(m_tempScore);
					m_tempScore = 0;
				}
			}
		}	
	}// ifǳ 
	//--------------------------------------------------------------------------------
	//! start  
	//! Enemy (,̴µ,,ū,DOORCLOSE) ,  Ѵ.
	//--------------------------------------------------------------------------------
	//! ٱ  浹ó
	//	m_x = 1130;	//! ?	AI  ġ 
	//	m_y = 640;
	switch(m_pEnemy->GetDirection())
	{
	case m_pEnemy->LEFT:
		break;
	case m_pEnemy->RIGHT:
		break;
	case m_pEnemy->UP:
		break;
	case m_pEnemy->DOWN:

		break;
	default:
		break;

	}
	if (m_pEnemy->GetX() < 128)
	{ //  ~!
		m_pEnemy->ImageRotation(m_pEnemy->UP);
		m_pEnemy->SetDirection(m_pEnemy->UP); //  ̵  ش.
	}
	if (m_pEnemy->GetX() >= (MAP_WIDTH)*128)
	{ // ~!
		m_pEnemy->ImageRotation(m_pEnemy->DOWN);
		m_pEnemy->SetDirection(m_pEnemy->DOWN); //  ̵  ش.
	}
	if (m_pEnemy->GetY() >= (MAP_HEIGHT)*128)
	{	//Ʒ ε.
		m_pEnemy->ImageRotation(m_pEnemy->RIGHT);
		m_pEnemy->SetDirection(m_pEnemy->RIGHT); //  ̵  ش.	}
	}else if (m_pEnemy->GetY() < 128)
	{	// ε.
		m_pEnemy->ImageRotation(m_pEnemy->LEFT);
		m_pEnemy->SetDirection(m_pEnemy->LEFT); //  ̵  ش.
	}

	//!  浹ó
	if (m_pEnemy != NULL)		
	{
		colWithEnemy = false;
		colWithEnemy = IsCrash(m_pHero->GetColRect(), m_pEnemy->GetColRect());
	}

	//!  
	if (m_pEnemy != NULL)	
	{
		if (m_pMap != NULL)	
		{
			for (int i=0; i<MAP_HEIGHT; i++)
			{
				for (int j=0; j<MAP_WIDTH; j++)
				{
					//! Rock ó ( ϵ)
					colWithTemp = false;
					if ((m_pMap->GetTile(2,i,j) == m_pMap->ROCK) || (m_pMap->GetTile(2,i,j) == m_pMap->ROCKMOVE))
					{
						colWithTemp = IsCrash(m_pEnemy->GetColRect(), m_pMap->GetColRect(2,i,j));
					}else if((m_pMap->GetTile(3,i,j) == m_pMap->TREESHORT) || (m_pMap->GetTile(3,i,j) == m_pMap->TREEBIG))
					{
						colWithTemp = IsCrash(m_pEnemy->GetColRect(), m_pMap->GetColRect(3,i,j));
					}
					if (colWithTemp)
					{
						switch (m_pEnemy->GetDirection())
						{
							case m_pEnemy->LEFT :
								m_pEnemy->AddX(react);
								m_pEnemy->ImageRotation(m_pEnemy->UP); // ̹  ش.
								m_pEnemy->SetDirection(m_pEnemy->UP); //  ̵  ش.
								break;
							case m_pEnemy->RIGHT :
								m_pEnemy->AddX(-react);
								m_pEnemy->ImageRotation(m_pEnemy->DOWN); // ̹  ش.
								m_pEnemy->SetDirection(m_pEnemy->DOWN); //  ̵  ش.

								break;
							case m_pEnemy->UP :
								m_pEnemy->AddY(react);
								m_pEnemy->ImageRotation(m_pEnemy->RIGHT); // ̹  ش.
								m_pEnemy->SetDirection(m_pEnemy->RIGHT); //  ̵  ش.
								break;
							case m_pEnemy->DOWN :
								m_pEnemy->AddY(-react);
								//! ̴  浹  1. ̹   ش. 2.   ش.
								m_pEnemy->ImageRotation(m_pEnemy->LEFT); // ̹  ش.
								m_pEnemy->SetDirection(m_pEnemy->LEFT); //  ̵  ش.
								break;
								default:
								break;
						}// switch 
					}// if 
				}// for 
			}// for 
		}// if 
	}


	//! ̴  

	//!---------------------------------------------------
	//! Enemy (, DOOROPEN,KEY) , Ѵ.
	//!---------------------------------------------------

	//! end .



}
// -----------------------------------------------------------------------
//! 浹ó Լ
bool CGameMain::IsCrash(CollisionRect* r_ColRect1, CollisionRect* r_ColRect2)
{	
	if (r_ColRect1->x2 > r_ColRect2->x1 && r_ColRect2->x2 > r_ColRect1->x1)
		if (r_ColRect1->y2 > r_ColRect2->y1 && r_ColRect2->y2 > r_ColRect1->y1)
			return true;

	return false;
}