//---------------------------------------------------------------------------//
//
//	Core - Timer
//
//	created : 2008/04/11
//
//---------------------------------------------------------------------------//

#pragma once
#pragma comment(lib, "winmm.lib")

#include <mmsystem.h>

class IBaseTimer;

//! ð  Ÿ̸ Ŭ
class CORE_API CTimer
{
public:
	enum eTimerType
	{
		TYPE_NONE = 0,
		TYPE_TIME_GET_TIME,
		TYPE_GET_TICK_COUNT,
		TYPE_HIGH_RESOLUTION,	// !! Warning !! :: Dual CPU Cannot use this mode.
	};

private:
	IBaseTimer*	m_pTimer;

	float	m_fElapsedTime;
	float	m_fFps;
	float	m_fPeriodCnt;
	float	m_fFpsCnt;

public:
	CTimer();
	~CTimer();

	bool	Init(eTimerType eType);
	void	Reset();
	void	Update();

	float	GetElapsedTime() { return m_fElapsedTime; }
	float	GetFps() { return m_fFps; }
};

//! Ÿ̸ ̽
class IBaseTimer
{
public:
	virtual ~IBaseTimer() { }

	virtual bool Init() = 0;
	virtual float Update() = 0;
};

//! timeGetTime Ÿ̸
class CTimeGetTimer : public IBaseTimer
{
private:
	DWORD	m_dwCurTime;
	DWORD	m_dwLastTime;
	float	m_fElapsedTime;

public:
	bool Init();
	float Update();
};

//! GetTickCount Ÿ̸
class CGetTickTimer : public IBaseTimer
{
private:
	DWORD	m_dwCurTime;
	DWORD	m_dwLastTime;
	float	m_fElapsedTime;

public:
	bool Init();
	float Update();
};

//! QueryPerformanceCounter Ÿ̸
class CHighResTimer : public IBaseTimer
{
private:
	LARGE_INTEGER	m_Frequency;
	LARGE_INTEGER	m_CurTick;
	LARGE_INTEGER	m_LastTick;
	float			m_fElapsedTime;

public:
	bool Init();
	float Update();
};

//! ð  Ŭ ̿Ͽ   ȭ Ŭ
class CORE_API CTimerSync
{
private:
	float m_fFps, m_fPeriodCnt, m_fFpsCnt;
	float m_fTimer, m_fAdjustTime, m_fFrameTime;
	float m_fSecPerFrame;
	float m_fdt, m_fRenderTime;

public:
	CTimerSync() { Clear(); }
	~CTimerSync() { }

	//! ʱȭ
	inline void Clear()
	{
		SetFps(60);
		m_fFps = 0.0f; m_fPeriodCnt = 0.0f; m_fFpsCnt = 0.0f;
		m_fTimer = 0.0f; m_fAdjustTime = 0.0f; m_fFrameTime = 0.0f;
		m_fdt = 0.0f; m_fRenderTime = 0.0f;
	}
	//! ϴ  Ѵ.
	inline void SetFps(int fps) 
	{
		m_fSecPerFrame = 1.0f / (float)fps;
	}
	inline bool Update(float dt)
	{
		m_fdt += dt;
		m_fTimer += dt;

		//! fps
		m_fPeriodCnt += dt;
		if (m_fPeriodCnt >= 1.0f)
		{
			m_fFps = m_fFpsCnt / m_fPeriodCnt;
			m_fFpsCnt = 0.0f;
			m_fPeriodCnt = 0.0f;
		}
		
		//!  Ӻ ʹ   ʰ ģ.
		if (m_fTimer <= 0.0f)
		{
			m_fAdjustTime += dt;
			return false;
		}
		else
		{
			m_fFpsCnt += 1.0f;

			m_fFrameTime = m_fAdjustTime + dt;
			m_fAdjustTime = 0.0f;

			m_fTimer -= m_fSecPerFrame;
			if (m_fTimer >= 2.0f) //! 2̻ Ʈ и  иȮ Ƿ ϴ ʱȭѴ.
				m_fTimer = 0.0f;

			return true;
		}
	}
	inline bool IsFrameSkip() 
	{
		if (m_fTimer > 0.0f)
		{
			return true;
		}
		else
		{
			m_fRenderTime = m_fdt;
			m_fdt = 0.0f;
			return false;
		}
	}
	inline float GetFrameTime() { return m_fFrameTime; }
	inline float GetFps() { return m_fFps; }
	inline DWORD GetSleepTime()
	{
		return (DWORD)((m_fSecPerFrame - m_fFrameTime) * 1000.0f);
	}
	//!  κ   ɸ ð ˾ƿ´.
	inline float GetRenderTime() { return m_fRenderTime; }
	inline float GetSecPerFrame() { return m_fSecPerFrame; }
};
