//---------------------------------------------------------------------------//
//
//	Framework - Input
//
//	created : 2008/04/17
//
//---------------------------------------------------------------------------//

#pragma once

#include "framework_config.h"
#include <dinput.h>

#pragma comment(lib, "dinput8.lib")
#pragma comment(lib, "dxguid.lib")

//---------------------------------------------------------------------------//
// Macros
//---------------------------------------------------------------------------//
#define	DI_USE_KEYBOARD			0x01
#define DI_USE_MOUSE			0x02
#define DI_USE_JOYSTICK			0x04

#define DIM_LEFT				0
#define DIM_RIGHT				1
#define DIM_CENTER				2
#define DIM_RESERVE 			3

#define KEY_WAITTIME			10

//! Ű Ŭ
class CKeyboard
{
private:
	LPDIRECTINPUTDEVICE8		m_pDevice;
	BYTE						m_keyStates[256];
	DWORD						m_dwKeyPressTime[256];
	bool						m_KeyEvent;
	DWORD						m_WaitInput;


	enum { DIK_NONE, DIK_RELEASE, DIK_PRESS, DIK_REPEAT };

public:
	CKeyboard();
	~CKeyboard();

	bool	CreateDevice(LPDIRECTINPUT8 lpDI, HWND hWnd, bool bExclusive, char* szErrorMsg);
	bool	Update();
	void	Release();

	inline bool	DIKNone(int key) { return (m_keyStates[key] == DIK_NONE); }
	inline bool	DIKRelease(int key) { return (m_keyStates[key] == DIK_RELEASE); }
	inline bool	DIKPress(int key) { return (m_keyStates[key] == DIK_PRESS); }
	inline bool	DIKRepeat(int key) { return (m_keyStates[key] == DIK_REPEAT || m_keyStates[key] == DIK_PRESS); }
	inline DWORD DIKGetPressTime(int key) { return m_dwKeyPressTime[key]; }
	inline bool DIKIsKeyEvent() { return m_KeyEvent; }
};

//! 콺 Ŭ
class CMouse
{
private:
	LPDIRECTINPUTDEVICE8		m_pDevice;
	DIMOUSESTATE				m_states;
	POINT						m_MousePos;
	POINT						m_PrevMousePos;
	bool						m_MouseEvent;

	enum { DIM_NONE, DIM_RELEASE, DIM_PRESS, DIM_REPEAT };

public:
	CMouse();
	~CMouse();

	bool		CreateDevice(LPDIRECTINPUT8 lpDI, HWND hWnd, bool bExclusive, char* szErrorMsg);
	bool		Update(HWND hWnd);
	void		Release();

	inline bool	DIMNone(int button) { return (m_states.rgbButtons[button] == DIM_NONE); }
	inline bool DIMRelease(int button) { return (m_states.rgbButtons[button] == DIM_RELEASE); }
	inline bool	DIMPress(int button) { return (m_states.rgbButtons[button] == DIM_PRESS); }
	inline bool	DIMRepeat(int button) { return (m_states.rgbButtons[button] == DIM_REPEAT); }

	inline void GetMousePos(int &mx, int &my) { mx=m_MousePos.x; my=m_MousePos.y; }
	inline void GetMousePos(HWND hWnd) { GetCursorPos(&m_MousePos); ScreenToClient(hWnd, &m_MousePos); }
	inline void	GetMovement(int &dx, int &dy, int &dz) { dx=m_states.lX; dy=m_states.lY; dz=m_states.lZ; }
	inline void GetPrevMousePos(int &mx, int &my) { mx=m_PrevMousePos.x; my=m_PrevMousePos.y; }
	inline bool	DIMIsMouseEvent() { return m_MouseEvent; }
	inline int DIMGetWheelMovement() { return -m_states.lZ/120; }
};

//! Է ġ ϴ ǲ Ŭ
class FRAMEWORK_API CInput : public CSingleton<CInput>
{
private:
	LPDIRECTINPUT8	m_pDI;
	CKeyboard		*m_pKeyboard;
	CMouse			*m_pMouse;
	HWND			m_hWnd;
	char			m_szErrorMsg[256];

public:
	CInput();
	~CInput();

	CKeyboard*		GetKeyboard() { if (m_pKeyboard) return m_pKeyboard; else return NULL; }
	CMouse*			GetMouse() { if (m_pMouse) return m_pMouse; else return NULL; }
	void			Update();
	bool			Init(HWND hWnd, HINSTANCE hInstance, bool bExclusive=false, DWORD flags=DI_USE_KEYBOARD);
	void			Release();
};

//! ǲ ý νϽ Ѵ. (Singleton)
FRAMEWORK_API CInput& GetInputSystem();

//   ζ Լ
FRAMEWORK_API inline bool DIKNone(int key)
{
	return (GetInputSystem().GetKeyboard()->DIKNone(key));
}

FRAMEWORK_API inline bool DIKRelease(int key)
{
	return (GetInputSystem().GetKeyboard()->DIKRelease(key));
}

FRAMEWORK_API inline bool DIKPress(int key)
{
	return (GetInputSystem().GetKeyboard()->DIKPress(key));
}

FRAMEWORK_API inline bool DIKRepeat(int key)
{
	return (GetInputSystem().GetKeyboard()->DIKRepeat(key));
}

FRAMEWORK_API inline DWORD DIKGetPressTime(int key)
{
	return (GetInputSystem().GetKeyboard()->DIKGetPressTime(key));
}

FRAMEWORK_API inline bool DIMNone(int button)
{
	return (GetInputSystem().GetMouse()->DIMNone(button));
}

FRAMEWORK_API inline bool DIMRelease(int button)
{
	return (GetInputSystem().GetMouse()->DIMRelease(button));
}

FRAMEWORK_API inline bool DIMPress(int button)
{
	return (GetInputSystem().GetMouse()->DIMPress(button));
}

FRAMEWORK_API inline bool DIMRepeat(int button)
{
	return (GetInputSystem().GetMouse()->DIMRepeat(button));
}

FRAMEWORK_API inline void DIMGetMovement(int &dx, int &dy, int &dz) 
{ 
	GetInputSystem().GetMouse()->GetMovement(dx, dy, dz);
}

FRAMEWORK_API inline void DIMGetMousePos(int &mx, int &my)
{
	GetInputSystem().GetMouse()->GetMousePos(mx, my);
}

FRAMEWORK_API inline void DIMGetPrevMousePos(int &mx, int &my)
{
	GetInputSystem().GetMouse()->GetPrevMousePos(mx, my);
}

FRAMEWORK_API inline int DIMGetWheelMovement()
{
	return GetInputSystem().GetMouse()->DIMGetWheelMovement();
}

FRAMEWORK_API inline bool DIMIsMouseEvent()
{
	return GetInputSystem().GetMouse()->DIMIsMouseEvent();
}

FRAMEWORK_API inline bool DIKIsKeyEvent()
{
	return GetInputSystem().GetKeyboard()->DIKIsKeyEvent();
}
