//---------------------------------------------------------------------------//
//
//	Framework - Win32 Application
//
//	Date : 2008/04/07
//
//---------------------------------------------------------------------------//

#include "framework_std.h"
#include "winapp.h"

using namespace std;

//  Ŭ  ν ֱ̾ static 
static CWindowApp *g_pStaticWnd = NULL;

// g_pStaticWnd Ͽ  Ŭ  ν  ִ static Լ.
LRESULT CWindowApp::StaticWndProc(HWND hWnd ,UINT Message, WPARAM wParam, LPARAM lParam)
{
	if (g_pStaticWnd != NULL)
	{
		return g_pStaticWnd->WndProc(hWnd, Message, wParam, lParam);
	}
	return 0;
}

//---------------------------------------------------------------------------//
//	Window Application Class
//---------------------------------------------------------------------------//

CWindowApp::CWindowApp() : m_hWnd(NULL), m_bLoop(true), m_bEnableIdle(false)
{
	g_pStaticWnd = this;
	memset(&m_WndClass, 0, sizeof(WNDCLASS));
	m_WndClass.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	m_WndClass.hInstance = (HINSTANCE)::GetModuleHandle(NULL);
	m_WndClass.lpfnWndProc = StaticWndProc;
	m_WndClass.cbClsExtra = 0;
	m_WndClass.cbWndExtra = 0;
}

CWindowApp::~CWindowApp()
{
	g_pStaticWnd = NULL;
}

//! 츦 ʱȭϰ ڿ ° Ѵ.
HRESULT CWindowApp::Create(const tstring& strAppName, const tstring& strClassName,
						int x, int y, int width, int height, 
						int style, HMENU hMenu, int icon, int cursor)
{
	m_strAppName = strAppName;
	m_strClassName = strClassName;

	if (cursor) m_WndClass.hCursor = LoadCursor(m_WndClass.hInstance, MAKEINTRESOURCE(cursor));
	else m_WndClass.hCursor = LoadCursor(0, IDC_ARROW);

	if (icon) m_WndClass.hIcon = LoadIcon(m_WndClass.hInstance, MAKEINTRESOURCE(icon));
	else m_WndClass.hIcon = LoadIcon(0, IDI_APPLICATION);

	if (hMenu) m_WndClass.lpszMenuName = MAKEINTRESOURCE(hMenu);
	else m_WndClass.lpszMenuName = NULL;

	m_WndClass.lpszClassName = strClassName.c_str();
	m_WndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);

	if (!RegisterClass(&m_WndClass))
	{
		MessageBox(NULL, _T(" Ŭ Ͽ Ͽϴ."), strAppName.c_str(), MB_ICONERROR);
		return false;
	}

	m_hWnd = CreateWindow(strClassName.c_str(), strAppName.c_str(), style, x, y, width, height, 
						  NULL, NULL, m_WndClass.hInstance, NULL);

	if (!m_hWnd) 
	{ 
		MessageBox(NULL, _T("  Ͽϴ."), strAppName.c_str(), MB_ICONERROR);
		return false;
	}

	ShowWindow(m_hWnd, SW_SHOW);
	UpdateWindow(m_hWnd);

	SetClientSize(width, height);

	SetActiveWindow(m_hWnd);

	return true;
}

//!  ޽ ν
LRESULT CWindowApp::WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_CHAR:
		{
#ifdef _DEBUG
			switch(wParam)
			{
			case VK_ESCAPE:
				{
					PostQuitMessage(0);
					m_bLoop = false;
				}break;
			}
#endif // _DEBUG
		}break;

	case WM_DESTROY:
		{
			m_bLoop = false;
		} break;

	case WM_SYSCOMMAND:
		{
			if ((wParam & 0xFFF0) == SC_SCREENSAVE || (wParam & 0xFFF0) == SC_MONITORPOWER)
			{
				return 0;
			}
		} break;
	}	
	return DefWindowProc(hWnd, Message, wParam, lParam);
}

//!  ޽ 
void CWindowApp::MsgLoop()
{
	MSG Msg;

	while(m_bLoop)
	{
		if (!m_bEnableIdle || PeekMessage(&Msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (!GetMessage(&Msg, NULL, 0, 0)) break;

			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}
		else if (m_bEnableIdle)
		{
			Idle();
		}
	}
}

//! Ŭ̾Ʈ  ũ⸦ width height Ѵ.
void CWindowApp::SetClientSize(int width, int height)
{
	RECT rcClient, rcWindow;

	GetClientRect(m_hWnd, &rcClient);
	GetWindowRect(m_hWnd, &rcWindow);

	int dx = (rcWindow.right-rcWindow.left) - (rcClient.right-rcClient.left);
	int dy = (rcWindow.bottom-rcWindow.top) - (rcClient.bottom-rcClient.top);

	RECT Rect = { 0, 0, width + dx, height + dy };

	SetWindowPos(m_hWnd, NULL, 0, 0, Rect.right - Rect.left, Rect.bottom - Rect.top,
				 SWP_NOZORDER | SWP_NOMOVE | SWP_NOACTIVATE);
}

//! 찡 ޽ óϰ  ð ̺κ óȴ.   η  ȴ.
void CWindowApp::Idle()
{
}

//!  ð Idle()  ȣ ϴ Լ.
void CWindowApp::SetIdle(bool bEnable)
{
	m_bEnableIdle = bEnable;
}

CWindowApp::operator HWND() const
{
	return m_hWnd;
}