//---------------------------------------------------------------------------//
//
//	Graphics - Renderer
//
//	Date : 2008/04/07
//
//---------------------------------------------------------------------------//

#include "renderer_interface.h"
#include "mathematics.h"
#include "texture.h"

#define MAX_ACTIVE_LIGHT	8

class GRAPHICS_API CRenderer : public IRenderDevice
{
private:
	LPDIRECT3D9				m_pD3D;
	LPDIRECT3DDEVICE9		m_pDevice;
	D3DDISPLAYMODE			m_d3ddm;
	D3DPRESENT_PARAMETERS	m_d3dpp;
	LPD3DXSPRITE			m_pSprite;
	LPD3DXFONT				m_pFont;
	LPDIRECT3DSURFACE9		m_pBackBuffer;	//!<   DC  ϱ  ȴ.
	bool					m_bFullScreen;

	matrix4		m_matWorld;
	matrix4		m_matView;
	matrix4		m_matProj;

	bool		m_bLighting;
	bool		m_bSpecularLighting;
	D3DLIGHT9	m_Light[MAX_ACTIVE_LIGHT];
	bool		m_bLightEnable[MAX_ACTIVE_LIGHT];

	bool		m_bZWriteEnable;
	bool		m_bZEnable;
	bool		m_bFogEnable;
	float		m_fFogStartDist;
	float		m_fFogEndDist;
	DWORD		m_dwFogColor;
	bool		m_bWireframe;

	bool		m_bAlphaTestEnable;
	BYTE		m_btAlphaTestRef;

	bool		m_bDrawTextureMode;
	DWORD		m_dwBackfaceCulling;
	DWORD		m_dwNumRenderedPrimitive;
	DWORD		m_dwNumCallDrawPrimitive;

public:
	CRenderer();
	~CRenderer();

	//!  ʱȭ Ѵ.
	/*!
		@param hWnd  ڵ
		@param nWidth   ũ
		@param nHeight   ũ
		@param bWindowMode   
		@return  ʱȭ 
	*/
	bool Init(HWND hWnd, int nWidth, int nHeight, bool bFullScreen);

	//!   Ѵ. Ҹڿ ڵ ȣȴ.
	void Release();

	//!   Ѵ. ()
	inline void SetView(const matrix4& matView);
	//!  Ǿִ   Ѵ.
	inline const matrix4& GetView() const;
	//!   Ѵ. (ġ, ,  )
	void SetView(const vector3& vecPos, const vector3& vecEye, const vector3& vecUp);
	//!   Ѵ.
	inline void SetProj(const matrix4& matProj);
	//!  Ǿִ   Ѵ.
	inline const matrix4& GetProj() const;
	//!   Ѵ.
	inline void SetWorld(const matrix4& matWorld);
	//!  Ǿִ   Ѵ.
	inline const matrix4& GetWorld() const;

	//! ۸ Ư  Ŭϰų ̹۸ Ŭ Ѵ.
    void ClearBuffer(bool bClearBackBuffer=true, DWORD dwBackBufferColor=0, bool bClearDepthBuffer=true);
	//!  ׸⸦ Ѵ. ( Ʈ ݵ BeginScene  EndScene ȿ ;Ѵ.)
	void BeginScene();
	//!  ׸⸦ . ( Ʈ ݵ BeginScene  EndScene ȿ ;Ѵ.)
	void EndScene();
	//! ڷ  ڵ 쿡 ۿ  ȭ ׸.
	void Present(HWND hWnd=NULL);

	//! ؽ  带 Ѵ. ( µ ؽ  忡 ° ȯȴ.)
	void BeginDrawTexture(int nWidth=0, int nHeight=0);
	//! 2D ǥ迡 ؽĸ Ѵ.
	void DrawTexture(CTexture* pTexture, int x, int y, DWORD dwColor=0xffffffff);
	//! ؽĸ Ѵ. ؽ κ Ҽ    Ҽ ִ.
	void DrawTextureEx(CTexture* pTexture, int sx, int sy, int width, int height, const vector3* pvecPos, DWORD dwColor=0xffffffff, const vector3* pvecCenter=NULL, const matrix4* pmatTransform=NULL);
	//! ؽĸ Ѵ. ؽ κ Ҽ  , ϰ ȸ ս Ҽ ִ.
	void DrawTextureEx(CTexture* pTexture, int sx, int sy, int width, int height, float dx, float dy, float cx=0.0f, float cy=0.0f, DWORD dwColor=0xffffffff, float fScale=1.0f, float fRadian=0.0f, bool bVertFlip=false, bool bHorzFlip=false);
	//! ؽĸ Ѵ. ؽ κ Ҽ  , ϰ ȸ ս Ҽ ִ.
	void DrawTextureEx(CTexture* pTexture, int sx, int sy, int width, int height, float dx, float dy, vector3& vecCenter, vector3& vecScale = vector3(1.0f, 1.0f, 1.0f), DWORD dwColor=0xffffffff, float fRadian=0.0f);
	//! ݱ  ؽĸ Ѵ.
	void FlushDrawTexture();
	//! ؽ  带 . ( µ 带 ϱ  ƿ´.)
	void EndDrawTexture();

	//! 2D ǥ迡 DC Ͽ ۾ Ѵ. (EndScene Ŀ  Ѵ.)
	void DrawTextDC(int x, int y, LPCTSTR cszText, COLORREF Color);

	//! Present  ϱ  ׸ Ƽ  ˾ƿ´.
	inline DWORD GetNumRenderedPrimitive();
	//! Present  ϱ  ȣ DrawPrimitive/DrawIndexedPrimitive  Ƚ ˾ƿ´.
	inline DWORD GetNumCallDrawPrimitive();

	//!   Ѵ. (ؽĳ ̽ ۿ ´  ϱ .)
	inline D3DFORMAT GetBackBufferFormat();
	//! D3D ġ ͸ Ѵ.
	inline LPDIRECT3DDEVICE9 GetDevice();

private:
	//! ũ 忡   ٸ Ѵ. (/Ǯũ)
	void SetScreenMode();
};

#include "renderer.inl"